from ignite.metrics.metric import Metric
from ignite.metrics.accuracy import Accuracy
from ignite.metrics.loss import Loss
from ignite.metrics.mean_absolute_error import MeanAbsoluteError
from ignite.metrics.mean_pairwise_distance import MeanPairwiseDistance
from ignite.metrics.mean_squared_error import MeanSquaredError
from ignite.metrics.epoch_metric import EpochMetric
from ignite.metrics.precision import Precision
from ignite.metrics.recall import Recall
from ignite.metrics.root_mean_squared_error import RootMeanSquaredError
from ignite.metrics.top_k_categorical_accuracy import TopKCategoricalAccuracy
from ignite.metrics.running_average import RunningAverage
from ignite.metrics.metrics_lambda import MetricsLambda
from ignite.metrics.confusion_matrix import ConfusionMatrix, IoU, mIoU, DiceCoefficient
from ignite.metrics.accumulation import VariableAccumulation, Average, GeometricAverage
from ignite.metrics.fbeta import Fbeta
from ignite.metrics.frequency import Frequency

__all__ = [
    'Metric',
    'Accuracy',
    'Loss',
    'MetricsLambda',
    'MeanAbsoluteError',
    'MeanPairwiseDistance',
    'MeanSquaredError',
    'ConfusionMatrix',
    'TopKCategoricalAccuracy',
    'Average',
    'DiceCoefficient',
    'EpochMetric',
    'Fbeta',
    'GeometricAverage',
    'IoU',
    'mIoU',
    'Precision',
    'Recall',
    'RootMeanSquaredError',
    'RunningAverage',
    'VariableAccumulation',
    'Frequency'
]



from typing import Callable, Any, Union
from enum import Enum

from ignite.engine import Engine
from ignite.engine.events import EventWithFilter, CallableEvents

from ignite.handlers.checkpoint import ModelCheckpoint, Checkpoint, DiskSaver
from ignite.handlers.timing import Timer
from ignite.handlers.early_stopping import EarlyStopping
from ignite.handlers.terminate_on_nan import TerminateOnNan

__all__ = [
    'ModelCheckpoint',
    'Checkpoint',
    'DiskSaver',
    'Timer',
    'EarlyStopping',
    'TerminateOnNan',
    'global_step_from_engine'
]


def global_step_from_engine(engine: Engine) -> Callable:
    """Helper method to setup `global_step_transform` function using another engine.
    This can be helpful for logging trainer epoch/iteration while output handler is attached to an evaluator.

    Args:
        engine (Engine): engine which state is used to provide the global step

    Returns:
        global step
    """

    def wrapper(_: Any, event_name: Union[EventWithFilter, CallableEvents, Enum]):
        return engine.state.get_event_attrib_value(event_name)

    return wrapper
